Package: Battlefield
Type: Package
Title: Swiss-army toolkit for selecting niche fronts and invasive
        margins in spatial transcriptomics data
Version: 1.0.0
Authors@R: c(
    person("Jean-Philippe","Villemin",email="jpvillemin@gmail.com",role=c("aut", "cre"),
    comment = c(ORCID = "0000-0002-1838-5880")),
    person("European Research Council",
         role = "fnd",
         comment = "ERC-2022"))
Description: Battlefield is a Swiss-army toolkit originally developed to define and extract spatial spots
 from specific tissue regions—such as front regions, niche borders, invasive margins, 
 and cluster interfaces—using spatial transcriptomics data or clustered tissue maps.
 It has since been extended to support trajectory selection and layer inspection, 
 and now provides a collection of low-level utilities for spatial transcriptomics analysis.
 These utilities are primarily intended to be reused within higher-level analytical packages.
 It is designed to work with sequencing-based platforms
 such as Visium at several resolutions and Visium HD(binned).
URL: https://github.com/ZheFrench/BattleField,
        https://zhefrench.github.io/Battlefield/
BugReports: https://github.com/ZheFrench/BattleField/issues
License: CeCILL | file LICENSE
Encoding: UTF-8
LazyData: FALSE
Depends: R (>= 4.6)
biocViews: Sequencing, Software, Transcriptomics, Spatial
Imports: stats, RANN, dplyr, SummarizedExperiment, methods
Suggests: BiocStyle, knitr, markdown, rmarkdown, SpatialExperiment,
        SpatialExperimentIO, VisiumIO, ggplot2, pheatmap, pals,
        OSTA.data, tidyr, STexampleData, testthat (>= 3.0.0),
        codetools, grid, tools
Config/testthat/edition: 3
RoxygenNote: 7.3.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/Battlefield
git_branch: RELEASE_3_23
git_last_commit: a0574ac
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:29:03 UTC; biocbuild
Author: Jean-Philippe Villemin [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1838-5880>),
  European Research Council [fnd] (ERC-2022)
Maintainer: Jean-Philippe Villemin <jpvillemin@gmail.com>
Built: R 4.6.0; ; 2026-04-29 04:34:07 UTC; unix
