# Generated by roxygen2: do not edit by hand

export(add_borders_to_spe)
export(add_layers_to_spe)
export(add_trajectories_to_spe)
export(adjacent_endpoint)
export(bresenham_line)
export(build_all_borders)
export(build_all_cores)
export(build_one_line)
export(build_one_trajectory)
export(build_similar_trajectories)
export(closest_spot)
export(compute_centroids)
export(count_all_inlaids)
export(count_all_neighborhoods)
export(count_inlaid)
export(count_neighborhood)
export(create_all_layers)
export(create_cluster_layers)
export(detect_grid_type)
export(directed_cluster_interface_pairs)
export(estimate_spot_spacing)
export(filter_out_by_endpoint_clusters)
export(get_inlaid_spots)
export(get_neighborhood_params)
export(get_neighborhood_spots)
export(point_segment_distance_vec)
export(remove_used_points)
export(select_border_spots)
export(select_core_spots)
export(shift_point)
export(unit_normal_left)
importFrom(RANN,nn2)
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,colData)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,slice_head)
importFrom(dplyr,slice_sample)
importFrom(dplyr,summarise)
importFrom(methods,is)
importFrom(stats,aggregate)
importFrom(stats,median)
importFrom(utils,head)
